//
//  ALGraphics.m
//  ALKit
//
//  Created by Aaron Levine on 5/9/13.
//  Copyright (c) 2013 Aaron Levine. All rights reserved.
//

#import "ALGraphics.h"

@implementation ALGraphics

@synthesize hue = _hue,
            saturation = _saturation,
            brightness = _brightness;

- (UIImage *)tabBarBackgroundImageWithRect:(CGRect)rect
{
        
    if (!_hue)
        _hue = ALKIT_TAB_HUE;
    if (!_saturation)
        _saturation = ALKIT_TAB_SATURATION;
    if(!_brightness)
        _brightness = ALKIT_TAB_BRIGHTNESS;
        
    CGFloat actualBrightness = self.brightness;
    
    CGContextRef context = createBitmapContext(rect.size.width, rect.size.height);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    
    CGFloat outerTopBrightness = 0.0*actualBrightness;
    
    UIColor * outerTop = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:outerTopBrightness alpha:1.0];
    
    UIColor * innerStroke = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:0.30*actualBrightness alpha:1.0];
    UIColor * innerTop = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:0.25*actualBrightness alpha:1.0];
    UIColor * innerBottom = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:1.0*actualBrightness alpha:1.0];
    
    CGFloat outerMargin = 0.0;
    CGRect outerRect = CGRectMake(0, 0, rect.size.width, rect.size.height - outerMargin);
    CGMutablePathRef outerPath = CGPathCreateMutable();
    CGPathAddRect(outerPath, nil, outerRect);
    
    CGFloat innerMargin = 0.0f;
    CGRect innerRect = CGRectInset(outerRect, innerMargin, innerMargin);
    CGMutablePathRef innerPath = CGPathCreateMutable();
    CGPathAddRect(innerPath, nil, innerRect);
    
    CGFloat highlightMargin = 0.0f;
    CGRect highlightRect = CGRectInset(outerRect, highlightMargin, highlightMargin);
    CGMutablePathRef highlightPath = CGPathCreateMutable();
    CGPathAddRect(highlightPath, nil, highlightRect);
    
    CGContextSaveGState(context);
    CGContextSetFillColorWithColor(context, outerTop.CGColor);
    CGContextAddPath(context, outerPath);
    CGContextFillPath(context);
    CGContextRestoreGState(context);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, innerPath);
    CGContextClip(context);
    drawGlossAndGradient(context, innerRect, innerTop.CGColor, innerBottom.CGColor);
    CGContextClip(context);
    CGContextRestoreGState(context);
    
    CGContextSaveGState(context);
    CGContextSetLineWidth(context, 2.0);
    CGContextSetStrokeColorWithColor(context, innerStroke.CGColor);
    CGContextAddPath(context, innerPath);
    CGContextClip(context);
    CGContextAddPath(context, innerPath);
    CGContextStrokePath(context);
    drawLinearGradient(context, innerRect, innerTop.CGColor, innerBottom.CGColor);
    CGContextRestoreGState(context);

    UIImage *image = [UIImage imageWithCGImage:CGBitmapContextCreateImage(context)];
    NSLog(@"%@", image.debugDescription);
    
    CFRelease(outerPath);
    CFRelease(innerPath);
    CFRelease(highlightPath);
    
    return [image resizableImageWithCapInsets:UIEdgeInsetsMake(5, 5, 5, 5)];
    
}

- (UIImage *)tabBarBackgroundSelectedImageWithRect:(CGRect)rect
{
 
    if (!_hue)
        _hue = ALKIT_TAB_HUE;
    if (!_saturation)
        _saturation = ALKIT_TAB_SATURATION;
    if(!_brightness)
        _brightness = ALKIT_TAB_BRIGHTNESS;
            
    CGContextRef context = createBitmapContext(rect.size.width * [[UIScreen mainScreen] scale], rect.size.height * [[UIScreen mainScreen] scale]);
    
    CGContextScaleCTM(context, [[UIScreen mainScreen] scale], [[UIScreen mainScreen] scale]);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    
    CGFloat outerMargin = 1.0;
    CGRect outerRect = CGRectInset(rect, outerMargin, outerMargin);
    CGMutablePathRef outerPath = CGPathCreateMutable();
    
    CGFloat innerMargin = 2.0f;
    CGRect innerRect = CGRectInset(outerRect, innerMargin, innerMargin);
    CGMutablePathRef innerPath = CGPathCreateMutable();
    CGPathAddEllipseInRect(innerPath, nil, innerRect);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, innerPath);
    CGContextClip(context);
    drawRadialGradient(context, innerRect, [[UIColor colorWithRed:1.0 green:0.8 blue:0.8 alpha:0.0] CGColor], [[UIColor clearColor] CGColor]);
    CGContextClip(context);
    CGContextRestoreGState(context);

    UIImage *image = [UIImage imageWithCGImage:CGBitmapContextCreateImage(context) scale:[[UIScreen mainScreen] scale] orientation:UIImageOrientationUp];
    NSLog(@"%@", image.debugDescription);
    
    CFRelease(outerPath);
    CFRelease(innerPath);
    
    return image;    
}


- (UIImage *)iconEmbededWithCGP:(NSString *)cgp Size:(CGFloat)size
{
    
    NSString *fileText = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:cgp ofType:@"txt"] encoding:NSUTF8StringEncoding error:nil];
    
    float outerSize = size + 10;
    
    size = size * ([[UIScreen mainScreen] scale] / 2);
    
    CGPathRef path = [ShapeUtilities createCGPathRefFromEPSString:fileText];
    CGContextRef context = createBitmapContext(outerSize, outerSize);
    CGRect bounds = CGPathGetBoundingBox(path);
        
    float scale = 0;
    if (bounds.size.width > bounds.size.height)
        scale = size/bounds.size.width;
    else
        scale = size/bounds.size.height;
    
    CGAffineTransform transformScale = CGAffineTransformMakeScale(scale, scale);
    path = CGPathCreateCopyByTransformingPath(path, &transformScale);
    
    bounds = CGPathGetBoundingBox(path);
    
    CGContextTranslateCTM(context, ((outerSize - bounds.size.width) - bounds.origin.x)/2, ((outerSize - bounds.size.height) - bounds.origin.y)/2);
    
    CGContextSaveGState(context);
    CGAffineTransform verticleTranslate = CGAffineTransformMakeTranslation(0, 3);
    CGPathRef shadowPath = CGPathCreateCopyByTransformingPath(path, &verticleTranslate);
    CGContextAddPath(context, shadowPath);
    CGPathRelease(shadowPath);
    CGContextSetFillColorWithColor(context, [[UIColor colorWithRed:0.2 green:0.2 blue:0.2 alpha:0.33] CGColor]);
    CGContextFillPath(context);
    CGContextRestoreGState(context);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, path);
    CGContextSetShadowWithColor(context, CGSizeMake(0, -1), 2, [[UIColor lightGrayColor] CGColor]);
    CGContextSetStrokeColorWithColor(context, [[UIColor colorWithHue:_hue saturation:_saturation/2 brightness:0.0 alpha:1.0] CGColor]);
    CGContextSetLineWidth(context, 1.0);
    CGContextStrokePath(context);
    CGContextRestoreGState(context);

    CGContextSaveGState(context);
    CGContextAddPath(context, path);
    CGContextClip(context);
    drawLinearGradient(context, bounds, [[UIColor colorWithHue:_hue saturation:_saturation/2 brightness:0.0 alpha:1.0] CGColor], [[UIColor colorWithHue:_hue saturation:_saturation/2 brightness:0.2 alpha:1.0] CGColor]);
    CGContextRestoreGState(context);
    
    CGImageRef imageRef = CGBitmapContextCreateImage(context);
    UIImage *image = [UIImage imageWithCGImage:imageRef scale:[[UIScreen mainScreen] scale] orientation:UIImageOrientationUp];
    CGImageRelease(imageRef);
    
    return image;
}

- (UIImage *)iconGlowingWithCGP:(NSString *)cgp Size:(CGFloat)size
{
    
    NSString *fileText = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:cgp ofType:@"txt"] encoding:NSUTF8StringEncoding error:nil];
    
    float outerSize = size + 10;
    
    size = size * ([[UIScreen mainScreen] scale] / 2);
    
    CGPathRef path = [ShapeUtilities createCGPathRefFromEPSString:fileText];
    CGContextRef context = createBitmapContext(outerSize, outerSize);
    CGRect bounds = CGPathGetBoundingBox(path);
    
    float scale = 0;
    if (bounds.size.width > bounds.size.height)
        scale = size/bounds.size.width;
    else
        scale = size/bounds.size.height;
    
    CGAffineTransform transformScale = CGAffineTransformMakeScale(scale, scale);
    path = CGPathCreateCopyByTransformingPath(path, &transformScale);
    
    bounds = CGPathGetBoundingBox(path);
    
    CGContextTranslateCTM(context, ((outerSize - bounds.size.width) - bounds.origin.x)/2, ((outerSize - bounds.size.height) - bounds.origin.y)/2);
        
    CGContextSaveGState(context);
    CGContextAddPath(context, path);
    CGContextSetShadowWithColor(context, CGSizeMake(0, 0), 4, [[UIColor colorWithHue:_hue saturation:_saturation brightness:0.8 alpha:1.0] CGColor]);
    CGContextSetStrokeColorWithColor(context, [[UIColor colorWithHue:_hue saturation:_saturation brightness:0.8 alpha:0.5] CGColor]);
    CGContextSetLineWidth(context, 1.0);
    CGContextStrokePath(context);
    CGContextRestoreGState(context);
    
    CGContextSaveGState(context);
    CGContextAddPath(context, path);
    CGContextClip(context);
    drawRadialGradient(context, bounds, [[UIColor whiteColor] CGColor], [[UIColor colorWithHue:_hue saturation:_saturation brightness:0.8 alpha:1.0] CGColor]);
    CGContextRestoreGState(context);
    
    CGImageRef imageRef = CGBitmapContextCreateImage(context);
    UIImage *image = [UIImage imageWithCGImage:imageRef scale:[[UIScreen mainScreen] scale] orientation:UIImageOrientationUp];
    CGImageRelease(imageRef);
    
    return image;
    
}

CGContextRef createBitmapContext (int pixelsWide,
                                    int pixelsHigh)
{
    CGContextRef    context = NULL;
    CGColorSpaceRef colorSpace;
    //void *          bitmapData;
    int             bitmapByteCount;
    int             bitmapBytesPerRow;
    
    bitmapBytesPerRow   = (pixelsWide * 4);
    bitmapByteCount     = (bitmapBytesPerRow * pixelsHigh);
    
    colorSpace = CGColorSpaceCreateDeviceRGB();
    context = CGBitmapContextCreate (NULL, pixelsWide, pixelsHigh, 8, bitmapBytesPerRow, colorSpace, kCGImageAlphaPremultipliedLast);
    
    CGColorSpaceRelease( colorSpace );
    
    if (context== NULL)
    {
        NSLog(@"Context not created!");
        return NULL;
    }
    
    return context;// 7
}

- (UIImage *) makeImageOutlineWithImage:(UIImage *)image
{
    CGImageRef imageRef = [image CGImage];
    
    size_t bItsPerPixel = CGImageGetBitsPerPixel(imageRef);
    size_t bYtesPerRow = CGImageGetBytesPerRow(imageRef);
    size_t bItsPerComponent = CGImageGetBitsPerComponent(imageRef);
    size_t height = CGImageGetHeight(imageRef);
    size_t width = CGImageGetWidth(imageRef);
    size_t bYtesPerPixel = bItsPerPixel / bItsPerComponent;
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    unsigned char *rawData = malloc(height * width * bYtesPerPixel);
    
    CGContextRef context = CGBitmapContextCreate(rawData, width, height, bItsPerComponent, bYtesPerRow, colorSpace, kCGImageAlphaPremultipliedLast);
    
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetShouldAntialias(context, YES);
    
    CGColorSpaceRelease(colorSpace);
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    CGImageRelease(imageRef);
    CGContextRelease(context);
    
    int byteIndex = 0;
    int columnIndex = 0;
    unsigned char *outlineData = malloc(height*width*bYtesPerPixel);
    
    for(int i = 0; i < height * width; i++)
    {
        int alpha = rawData[byteIndex + 3];
        
        if (columnIndex == width)
            columnIndex = 0;
        
        if (alpha > 0) {
            
            if (columnIndex == 0 || columnIndex == width - 1) {
                outlineData[byteIndex + 3] = 255;
            }
            else if (i < width || i > (width*height) - width) {
                outlineData[byteIndex + 3] = 255;
            }
            else {
                for (int row = -1; row <= 1; row++) {
                    for (int column = -1; column <= 1; column++) {
                        if (row != 0 && column != 0) {
                            int testIndex = byteIndex + (4 * (column + (row * width) ));
                            if (rawData[testIndex + 3] == 0) {
                                outlineData[byteIndex + 3] = 255;
                                row = 2;
                                column = 2;
                            }
                            else
                                outlineData[byteIndex + 3] = 0;
                        }
                    }
                }
            }
            
        }
                
        columnIndex++;
        byteIndex += 4;
    }
    
    context = CGBitmapContextCreate(outlineData, width, height, bItsPerComponent, bYtesPerRow, colorSpace, kCGImageAlphaPremultipliedLast);
    
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetShouldAntialias(context, YES);
    
    CGImageRef outline = CGBitmapContextCreateImage(context);
    
    CGContextRelease(context);
    
    UIImage *outlineImage = [UIImage imageWithCGImage:outline];
    CGImageRelease(outline);
    
    return outlineImage;
}

- (UIImage *) changeImageColor:(UIImage *)image Red:(float)red Green:(float)green Blue:(float)blue
{
    CGImageRef imageRef = [image CGImage];
    
    size_t bItsPerPixel = CGImageGetBitsPerPixel(imageRef);
    size_t bYtesPerRow = CGImageGetBytesPerRow(imageRef);
    size_t bItsPerComponent = CGImageGetBitsPerComponent(imageRef);
    size_t height = CGImageGetHeight(imageRef);
    size_t width = CGImageGetWidth(imageRef);
    size_t bYtesPerPixel = bItsPerPixel / bItsPerComponent;
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    float scale = [[UIScreen mainScreen] scale];
    unsigned char *rawData = malloc(height*width*bYtesPerPixel*scale);
    unsigned char *outData = malloc(height*width*bYtesPerPixel*scale);
    
    CGContextRef context = CGBitmapContextCreate(rawData, width, height, bItsPerComponent, bYtesPerRow, colorSpace, kCGImageAlphaPremultipliedLast);
    CGContextScaleCTM(context, scale, scale);
            
    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetShouldAntialias(context, YES);
    
    CGContextDrawImage(context, CGRectMake(0, 0, width / scale, height / scale), imageRef);
    
    CGContextRelease(context);
    
    int byteIndex = 0;
        
    for(int i = 0; i < height * width; i++)
    {
        int alpha = rawData[byteIndex + 3];
        
        if (alpha > 0) {
            
            outData[byteIndex] = red - rawData[byteIndex];
            outData[byteIndex + 1] = green - rawData[byteIndex + 1];
            outData[byteIndex + 2] = blue - rawData[byteIndex + 2];
            outData[byteIndex + 3] = 255 - rawData[byteIndex + 3];
            
        }
        else
        {
            outData[byteIndex] = rawData[byteIndex];
            outData[byteIndex + 1] = rawData[byteIndex + 1];
            outData[byteIndex + 2] = rawData[byteIndex + 2];
            outData[byteIndex + 3] = rawData[byteIndex + 3];

        }
        byteIndex += 4;
    }
    
    context = CGBitmapContextCreate(outData, width, height, bItsPerComponent, bYtesPerRow, colorSpace,kCGImageAlphaPremultipliedLast);
    
    CGColorSpaceRelease(colorSpace);
    CGContextScaleCTM(context, scale, scale);

    CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
    CGContextSetAllowsAntialiasing(context, YES);
    CGContextSetShouldAntialias(context, YES);
    
    CGImageRef output = CGBitmapContextCreateImage(context);
        
    CGContextRelease(context);
    
    return [UIImage imageWithCGImage:output scale:scale orientation:UIImageOrientationUp];
}

- (CGMutablePathRef)createRoundedRectForRect:(CGRect)rect Radius:(CGFloat)radius Panel:(NSInteger)panel
{
    CGMutablePathRef path = CGPathCreateMutable();
    
    switch (panel) {

        case ALPanelPositionUnbounded:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionCenter:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionLeft:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionRight:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionTop:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionBottom:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionTopLeft:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), radius);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionTopRight:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionBottomLeft:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;

        case ALPanelPositionBottomRight:
            CGPathMoveToPoint(path, NULL, CGRectGetMidX(rect), CGRectGetMinY(rect));
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMaxY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMaxX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMaxY(rect), radius);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMaxY(rect), CGRectGetMinX(rect), CGRectGetMinY(rect), 0);
            CGPathAddArcToPoint(path, NULL, CGRectGetMinX(rect), CGRectGetMinY(rect), CGRectGetMaxX(rect), CGRectGetMinY(rect), 0);
            CGPathCloseSubpath(path);
            break;
        default:
            break;
    }
    
    return path;
}

void drawGlossAndGradient(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef endColor)
{
    drawLinearGradient(context, rect, startColor, endColor);
    
    UIColor *glossColor1 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.35];
    UIColor *glossColor2 = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.1];
    
    CGRect topHalf = CGRectMake(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height/2);
    
    drawLinearGradient(context, topHalf, glossColor1.CGColor, glossColor2.CGColor);
}

void drawLinearGradient(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef endColor)
{
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGFloat locations[] = {0.0, 1.0};
    NSArray *colors = @[(__bridge id)startColor, (__bridge id)endColor];
    
    CGGradientRef gradient = CGGradientCreateWithColors(colorSpace, (__bridge CFArrayRef)colors, locations);
    
    CGPoint startPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMinY(rect));
    CGPoint endPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMaxY(rect));
    
    CGContextSaveGState(context);
    CGContextAddRect(context, rect);
    CGContextDrawLinearGradient(context, gradient, startPoint, endPoint, 0);
    
    CGGradientRelease(gradient);
    CGColorSpaceRelease(colorSpace);
}

void drawRadialGradient(CGContextRef context, CGRect rect, CGColorRef startColor, CGColorRef endColor)
{
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    CGFloat locations[] = {0.0, 1.0};
    NSArray *colors = @[(__bridge id)startColor, (__bridge id)endColor];
    
    CGGradientRef gradient = CGGradientCreateWithColors(colorSpace, (__bridge CFArrayRef)colors, locations);
    
    CGPoint startPoint = CGPointMake(CGRectGetMidX(rect), CGRectGetMidY(rect));
    
    CGContextSaveGState(context);
    CGContextAddRect(context, rect);
    CGContextDrawRadialGradient(context, gradient, startPoint, 0, startPoint, CGRectGetMaxX(rect), 0);
    
    CGGradientRelease(gradient);
    CGColorSpaceRelease(colorSpace);
}

void drawLinearFade(CGContextRef context, CGRect rect, CGColorRef startColor)
{
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    float domain[2] = {0,1};
    float range[8] = {0,1,0,1,0,1,0,1};
    CGFunctionCallbacks callbacks = { 0, evaluate, NULL };
    CGFunctionRef function = CGFunctionCreate(nil, 2, domain , 8, range , &callbacks);
    
    CGPoint startPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMinY(rect));
    CGPoint endPoint = CGPointMake(CGRectGetMinX(rect), CGRectGetMaxY(rect));
    CGShadingRef shader = CGShadingCreateAxial(colorSpace, startPoint, endPoint, function , YES, YES);
    CGContextSaveGState(context);
    CGContextDrawShading(context, shader);
    
    startPoint = CGPointMake(CGRectGetMinX(rect) + CGRectGetMaxY(rect), CGRectGetMinY(rect));
    endPoint = CGPointMake(CGRectGetMinX(rect) + CGRectGetMaxY(rect), CGRectGetMaxY(rect));
    shader = CGShadingCreateAxial(colorSpace, startPoint, endPoint, function , NO, NO);
    CGContextDrawShading(context, shader);
    
    CGFunctionRelease(function);
    CGShadingRelease(shader);
    CGColorSpaceRelease(colorSpace);
}

static void evaluate (void *info, const float *in, float *out)
{
    float thing;
    thing = in[0];
    
    out[0] = 0.5;
    out[1] = 0.5;
    out[2] = 0.5;
    out[3] = 1 - in[0];
    
}

@end
