//
//  ALPanelButton.m
//  ALKit
//
//  Created by Aaron Levine on 5/5/13.
//  Copyright (c) 2013 Aaron Levine. All rights reserved.
//

#import "ALPanelButton.h"

@implementation ALPanelButton

@synthesize hue = _hue,
            saturation = _saturation,
            brightness = _brightness,
            radius = _radius,
            panel = _panel,
            hasShadow = _hasShadow;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChange) name:UIDeviceOrientationDidChangeNotification object:nil];
        
        self.opaque = NO;
        self.backgroundColor = [UIColor clearColor];
        self.clipsToBounds = NO;
        
        [self.imageView setContentMode:UIViewContentModeCenter];
        [self.imageView setClipsToBounds:NO];
        
        UIImage *fontGradientSelected = [UIImage imageNamed:@"Font_Gradient_Selected.png"];
        [self setTitleColor:[UIColor colorWithPatternImage:fontGradientSelected] forState:UIControlStateHighlighted];
        
        if (!_hue)
            _hue = ALPANEL_DEFAULT_HUE;
        if (!_saturation)
            _saturation = ALPANEL_DEFAULT_SATURATION;
        if(!_brightness)
            _brightness = ALPANEL_DEFAULT_BRIGHTNESS;
        if (!_radius)
            _radius = ALPANEL_DEFAULT_RADIUS;
        if (!_panel)
            _panel = ALPANEL_DEFAULT_PANEL;
        if (!_hasShadow)
            _hasShadow = NO;
        if (!_sbHue)
            _sbHue = -1;
        if (!_sbSaturation)
            _sbSaturation = -1;
        if (!_sbBrightness)
            _sbBrightness = -1;
    }
    return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    
    if (self == [super initWithCoder:aDecoder]) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChange) name:UIDeviceOrientationDidChangeNotification object:nil];
        
        self.opaque = NO;
        self.backgroundColor = [UIColor clearColor];
        self.clipsToBounds = NO;
        
        [self.imageView setContentMode:UIViewContentModeCenter];
        [self.imageView setClipsToBounds:NO];
        
        UIImage *fontGradientSelected = [UIImage imageNamed:@"Font_Gradient_Selected.png"];
        [self setTitleColor:[UIColor colorWithPatternImage:fontGradientSelected] forState:UIControlStateHighlighted];
        
        if (!_hue)
            _hue = ALPANEL_DEFAULT_HUE;
        if (!_saturation)
            _saturation = ALPANEL_DEFAULT_SATURATION;
        if(!_brightness)
            _brightness = ALPANEL_DEFAULT_BRIGHTNESS;
        if (!_radius)
            _radius = ALPANEL_DEFAULT_RADIUS;
        if (!_panel)
            _panel = ALPANEL_DEFAULT_PANEL;
        if (!_hasShadow)
            _hasShadow = NO;
        if (!_sbHue)
            _sbHue = -1;
        if (!_sbSaturation)
            _sbSaturation = -1;
        if (!_sbBrightness)
            _sbBrightness = -1;
    }
    return self;
}

- (void)drawRect:(CGRect)rect
{    
    if (_sbHue != -1)
        _hue = _sbHue/255;
    if (_sbSaturation != -1)
        _saturation = _sbSaturation/100;
    if(_sbBrightness != -1)
        _brightness = _sbBrightness/100;
    
    if (_panel != -1) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChange) name:UIDeviceOrientationDidChangeNotification object:nil];
        
        CGFloat actualBrightness = self.brightness;
        if (self.state == UIControlStateHighlighted || self.state == UIControlStateSelected) {
            actualBrightness -= 0.20;
        }
        
        CGContextRef context = UIGraphicsGetCurrentContext();
        CGContextSetAllowsAntialiasing(context, YES);
        CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
        
        UIColor *blackColor = [UIColor colorWithRed:0.8 green:0.8 blue:0.8 alpha:1.0];
        UIColor *highlightStart = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.4];
        UIColor *highlightStop = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:0.1];
        UIColor *shadowColor = [UIColor colorWithRed:0.2 green:0.2 blue:0.2 alpha:0.5];
        
        CGFloat outerTopBrightness = 0.8*actualBrightness;
        CGFloat outerBottomBrightness = 1.0*actualBrightness;
        if (self.state == UIControlStateNormal){
            outerTopBrightness = 1.0*actualBrightness;
            outerBottomBrightness = 0.8*actualBrightness;
        }
        
        
        UIColor * outerTop = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:outerTopBrightness alpha:1.0];
        UIColor * outerBottom = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:outerBottomBrightness alpha:1.0];
        
        
        UIColor * innerStroke = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:0.70*actualBrightness alpha:1.0];
        UIColor * innerTop = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:1.0*actualBrightness alpha:1.0];
        UIColor * innerBottom = [UIColor colorWithHue:self.hue saturation:self.saturation brightness:0.90*actualBrightness alpha:1.0];
        
        CGFloat outerMargin = 0.0;
        if (_hasShadow)
            outerMargin = 4.0f;
        CGRect outerRect = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height - outerMargin); //CGRectInset(rect, 0, outerMargin);
        CGMutablePathRef outerPath = [[ALGraphics alloc] createRoundedRectForRect:outerRect Radius:self.radius Panel:self.panel];
        
        CGFloat innerMargin = 2.0f;
        CGRect innerRect = CGRectInset(outerRect, innerMargin, innerMargin);
        CGMutablePathRef innerPath = [[ALGraphics alloc] createRoundedRectForRect:innerRect Radius:self.radius Panel:self.panel];
        
        CGFloat highlightMargin = 2.0f;
        CGRect highlightRect = CGRectInset(outerRect, highlightMargin, highlightMargin);
        CGMutablePathRef highlightPath = [[ALGraphics alloc] createRoundedRectForRect:highlightRect Radius:self.radius Panel:self.panel];

        if (self.state != UIControlStateHighlighted && self.state != UIControlStateSelected) {
            CGContextSaveGState(context);
            CGContextSetFillColorWithColor(context, outerTop.CGColor);
            if (_hasShadow)
                CGContextSetShadowWithColor(context, CGSizeMake(0, 2), 3.0, shadowColor.CGColor);
            CGContextAddPath(context, outerPath);
            CGContextFillPath(context);
            CGContextRestoreGState(context);
        }
            
        CGContextSaveGState(context);
        CGContextAddPath(context, outerPath);
        CGContextClip(context);
        drawGlossAndGradient(context, outerRect, outerTop.CGColor, outerBottom.CGColor);
        CGContextRestoreGState(context);
        
        CGContextSaveGState(context);
        CGContextAddPath(context, innerPath);
        CGContextClip(context);
        drawGlossAndGradient(context, innerRect, innerTop.CGColor, innerBottom.CGColor);
        CGContextClip(context);
        CGContextRestoreGState(context);
        
        if (self.state != UIControlStateHighlighted && self.state != UIControlStateSelected) {
            CGContextSaveGState(context);
            CGContextSetLineWidth(context, 1.0);
            CGContextAddPath(context, outerPath);
            CGContextAddPath(context, highlightPath);
            CGContextEOClip(context);
            drawLinearGradient(context, outerRect, highlightStart.CGColor, highlightStop.CGColor);
            CGContextRestoreGState(context);
        }
        
        CGContextSaveGState(context);
        CGContextSetLineWidth(context, 1.0);
        CGContextSetStrokeColorWithColor(context, blackColor.CGColor);
        CGContextAddPath(context, outerPath);
        CGContextStrokePath(context);
        CGContextRestoreGState(context);
        
        CGContextSaveGState(context);
        CGContextSetLineWidth(context, 2.0);
        CGContextSetStrokeColorWithColor(context, innerStroke.CGColor);
        CGContextAddPath(context, innerPath);
        CGContextClip(context);
        CGContextAddPath(context, innerPath);
        CGContextStrokePath(context);
        drawLinearGradient(context, innerRect, innerTop.CGColor, innerBottom.CGColor);
        CGContextRestoreGState(context);
        
        CFRelease(outerPath);
        CFRelease(innerPath);
        CFRelease(highlightPath);
    }
}

- (void)setHighlighted:(BOOL)highlighted{
    [super setHighlighted:highlighted];
    [self setNeedsDisplay];
}

- (void)orientationChange
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self setNeedsDisplay];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)setHue:(CGFloat)hue
{    
    _hue = hue;
    [self setNeedsDisplay];
}

- (void)setSaturation:(CGFloat)saturation
{
    _saturation = saturation;
    [self setNeedsDisplay];
}

- (void)setBrightness:(CGFloat)brightness
{
    _brightness = brightness;
    [self setNeedsDisplay];
}
- (void)setRadius:(CGFloat)radius
{
    _radius = radius;
    [self setNeedsDisplay];
}
- (void)setPanel:(NSInteger)panel
{
    _panel = panel;
    [self setNeedsDisplay];
}

@end
