//
//  ALPopup.m
//  ALKit
//
//  Created by Aaron Levine on 5/30/13.
//  Copyright (c) 2013 Aaron Levine. All rights reserved.
//

#import "ALPopup.h"

#define DegreesToRadians(degrees) (degrees * M_PI / 180)

@implementation ALPopup

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        [self setOpaque:NO];
        [self setClipsToBounds:YES];
    }
    return self;
    
}

- (id)initWithText:(NSString *)text Buttons:(NSString *)buttons, ...
{
    if (!_width)
        _width = ALPOPUP_WIDTH;
    if (!_hue)
        _hue = ALPOPUP_HUE;
    if (!_radius)
        _radius = ALPOPUP_RADIUS;
    if (!_saturation)
        _saturation = ALPOPUP_SATURATION;
    if (!_brightness)
        _brightness = ALPOPUP_BRIGHTNESS;
    if (!_pAlpha)
        _pAlpha = ALPOPUP_ALPHA;
    if (!_font)
        _font = ALPOPUP_FONT;
    if (!_fontSize)
        _fontSize = ALPOPUP_FONT_SIZE;
    
    NSMutableDictionary *textAttribs = [[NSMutableDictionary alloc] init];
    NSMutableParagraphStyle *pStyle = [[NSMutableParagraphStyle alloc] init];
    [pStyle setAlignment:NSTextAlignmentCenter];
    [textAttribs setObject:pStyle forKey:NSParagraphStyleAttributeName];
    [textAttribs setObject:[UIColor whiteColor] forKey:NSForegroundColorAttributeName];
    [textAttribs setObject:[UIFont fontWithName:self.font size:self.fontSize] forKey:NSFontAttributeName];
    
    NSAttributedString *attribText = [[NSAttributedString alloc] initWithString:text attributes:textAttribs];
    self.textFrame = CGRectMake(5, 5, (self.width - 10), 0);
    self.textView = [[AlTextView alloc] initWithFrame:self.textFrame];
    [self.textView setText:attribText];
    [self.textView setHeight:self.textView.estimatedHeight];
    self.textFrame = CGRectMake(5, 5, (self.width - 10), self.textView.height + 5);
    
    CGFloat buttonHeight = 55;
    if (buttons) {
        
        buttonHeight = 60;
        
        id buttonTitle = nil;
        int buttonIndex = 0;
        int buttonTotal = 1;
        int buttonRow = 0;
        NSMutableArray *buttonsMutable = [[NSMutableArray alloc] init];
        
        va_list argumentList;
        va_start(argumentList, buttons);
        
        while ((buttonTitle = (__bridge NSString *)va_arg(argumentList, void *))) {
            buttonTotal++;
            buttonIndex++;
            if (buttonIndex == 3) {
                buttonRow++;
                buttonIndex = 0;
            }
        }
        
        int lastRow = buttonRow;
        int lastTotal = buttonIndex + 1;
        
        buttonIndex = 0;
        buttonRow = 0;
        
        NSLog(@"%i", buttonTotal);
        
        va_end(argumentList);
        
        ALPanelButton *button;
        
        if (buttonRow != lastRow) {
            int buttonWidth = (self.width / 3) - 10;
            button = [[ALPanelButton alloc] initWithFrame:CGRectMake(7 + (7 * buttonIndex) + (buttonWidth * buttonIndex), 6 + (50*buttonRow) + self.textView.height + 5, buttonWidth, 44)];
        }
        else if (buttonRow == lastRow) {
            int buttonWidth = (self.width / lastTotal) - 10;
            button = [[ALPanelButton alloc] initWithFrame:CGRectMake(7 + (7 * buttonIndex) + (buttonWidth * buttonIndex), 6 + (50*buttonRow) + self.textView.height + 5, buttonWidth - ((3 - lastTotal) * 3), 44)];
        }
        [button setTitle:buttons forState:(UIControlState)nil];
        [[button titleLabel] setFont:[UIFont fontWithName:self.font size:self.fontSize]];
        [button setAlpha:self.pAlpha];
        button.tag = buttonIndex + 3*buttonRow;
        [button addTarget:self action:@selector(buttonPressed:) forControlEvents:UIControlEventTouchUpInside];
        [buttonsMutable addObject:button];
    
        va_start(argumentList, buttons);
        
        buttonRow = 0;
        
        while ((buttonTitle = (__bridge NSString *)va_arg(argumentList, void *))) {
            buttonIndex++;
            if (buttonIndex == 3) {
                buttonHeight += 50;
                buttonIndex = 0;
                buttonRow++;
            }
            
            ALPanelButton *button;
            
            if (buttonRow != lastRow) {
                int buttonWidth = (self.width / 3) - 10;
                button = [[ALPanelButton alloc] initWithFrame:CGRectMake(7 + (7 * buttonIndex) + (buttonWidth * buttonIndex), 6 + (50*buttonRow) + self.textView.height + 5, buttonWidth, 44)];
            }
            else if (buttonRow == lastRow) {
                int buttonWidth = (self.width / lastTotal) - 10;
                button = [[ALPanelButton alloc] initWithFrame:CGRectMake(7 + (7 * buttonIndex) + (buttonWidth * buttonIndex), 6 + (50*buttonRow) + self.textView.height + 5, buttonWidth - ((3 - lastTotal) * 3), 44)];
            }
            [button setTitle:buttonTitle forState:(UIControlState)nil];
            [[button titleLabel] setFont:[UIFont fontWithName:self.font size:self.fontSize]];
            [button setAlpha:self.pAlpha];
            button.tag = buttonIndex + 3*buttonRow;
            [button addTarget:self action:@selector(buttonPressed:) forControlEvents:UIControlEventTouchUpInside];
            [buttonsMutable addObject:button];
        }
        self.buttons = [[NSArray alloc] initWithArray:buttonsMutable];
        va_end(argumentList);
    }
    NSLog(@"button height: %f", buttonHeight);
    
    self = [self initWithFrame:CGRectMake(0, 0, self.width, buttonHeight + self.textView.height + 10)];
    self.backgroundColor = [UIColor clearColor];
        
    return self;
}

- (void)setWidth:(CGFloat)width
{
    _width = width;
    [self setNeedsDisplay];
}

- (void)setHue:(CGFloat)hue
{
    _hue = hue;
    [self setNeedsDisplay];
}

- (void)setRadius:(CGFloat)radius
{
    _radius = radius;
    [self setNeedsDisplay];
}

- (void)setSaturation:(CGFloat)saturation
{
    _saturation = saturation;
    [self setNeedsDisplay];
}

- (void)setBrightness:(CGFloat)brightness
{
    _brightness = brightness;
    [self setNeedsDisplay];
}

- (void)setPAlpha:(CGFloat)pAlpha
{
    _pAlpha = pAlpha;
    [self setNeedsDisplay];
}

- (void)setFont:(NSString *)font
{
    _font = font;
    [self setNeedsDisplay];
}

- (void)setFontSize:(CGFloat)fontSize
{
    _fontSize = fontSize;
    [self setNeedsDisplay];
}

- (void)show
{
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChange:) name:UIApplicationDidChangeStatusBarOrientationNotification object:nil];
    
    self.window.windowLevel = UIWindowLevelAlert;
    self.window.backgroundColor = [UIColor clearColor];
    self.center = CGPointMake(CGRectGetMidX(self.window.bounds), CGRectGetMidY(self.window.bounds));
    
    CGRect startRect = CGRectMake(self.center.x, self.center.y, 0, 0);
    CGRect endRect = self.frame;
    
    self.frame = startRect;
    [self setAlpha:0.0];
    
    [self.window addSubview:self];
    [self.window makeKeyAndVisible];
    
    [UIView animateWithDuration:0.2 animations:^{
        self.frame = endRect;
        [self setAlpha:1.0];
    }];
}

- (void)hide
{
    CGRect endRect = CGRectMake(self.center.x, self.center.y, 0, 0);
    
    [UIView animateWithDuration:0.2 animations:^{
        self.frame = endRect;
        [self setAlpha:0.0];
    } completion:^(BOOL finished) {
        self.window.hidden = YES;
        self.window = nil;
    }];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    
    ALPanelCell *popupBody = [[ALPanelCell alloc] initWithFrame:self.bounds];
    [popupBody setAlpha:self.pAlpha];
    [popupBody setHue:self.hue];
    [popupBody setRadius:self.radius];
    [popupBody setSaturation:self.saturation];
    [popupBody setBrightness:self.brightness];
    [popupBody setHasShadow:YES];
    
    UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
    [self setTransform:[self transformForOrientation:orientation]];
    [self addSubview:popupBody];
    
    self.textView.frame = CGRectMake(0, 0, self.textView.frame.size.width, self.textView.frame.size.height);
    self.textFrame = CGRectMake(5, 5, self.textFrame.size.width, self.textFrame.size.height);
    
    UILabel *label = [[UILabel alloc] initWithFrame:self.textFrame];
    [label setAttributedText:self.textView.text];
    [label setBackgroundColor:[UIColor clearColor]];
    [label setNumberOfLines:0];
    [label setLineBreakMode:NSLineBreakByWordWrapping];
    [self addSubview:label];
    
    for (ALPanelButton *button in self.buttons) {
        [button setHue:self.hue];
        [button setRadius:self.radius];
        [button setSaturation:self.saturation];
        [button setBrightness:self.brightness];
        [self addSubview:button];
    }
    
}

- (void)orientationChange:(NSNotification *)notification
{
    UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
    [self setTransform:[self transformForOrientation:orientation]];
}

- (CGAffineTransform)transformForOrientation:(UIInterfaceOrientation)orientation {
    switch (orientation) {
        case UIInterfaceOrientationLandscapeLeft:
            return CGAffineTransformMakeRotation(-DegreesToRadians(90));
        case UIInterfaceOrientationLandscapeRight:
            return CGAffineTransformMakeRotation(DegreesToRadians(90));
        case UIInterfaceOrientationPortraitUpsideDown:
            return CGAffineTransformMakeRotation(DegreesToRadians(180));
        case UIInterfaceOrientationPortrait:
        default:
            return CGAffineTransformMakeRotation(DegreesToRadians(0));
    }
}

- (void)buttonPressed:(id)sender
{
    ALPanelButton *button = sender;
    [self.delegate popup:self didDismissWithIndex:button.tag];
    [self hide];
}


//- (void)drawRect:(CGRect)rect
//{
//    // Drawing code
//    [super drawRect:rect];
//}

@end
