//
//  AlTextView.m
//  ALKit
//  Created by Aaron Levine on 5/22/13.
//  Copyright (c) 2013 Aaron Levine. All rights reserved.
//

#import "AlTextView.h"

@implementation AlTextView

//@synthesize path, text;
@synthesize delegate;

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self setOpaque:NO];
        [self setBackgroundColor:[UIColor clearColor]];
        self.path = CGPathCreateWithRect(frame, nil);
        self.text = [[NSAttributedString alloc] init];
        self.width = frame.size.width;
    }
    return self;
}

- (id)initWithPath:(CGPathRef)initPath
{
    CGRect frame = CGPathGetBoundingBox(initPath);
    self = [super initWithFrame:frame];
    if (self) {
        self.path = initPath;
    }
    return self;
}

- (void)setPath:(CGPathRef)newPath
{
    _path = newPath;
    self.frame = CGPathGetPathBoundingBox(newPath);
    [self setNeedsDisplay];
}

- (void)setHeight:(CGFloat)height
{
    _height = height;
    self.bounds = CGRectMake(0, 0, self.bounds.size.width, height);
}

- (CGFloat)estimatedHeight
{
    
    CTFramesetterRef frameSetter = CTFramesetterCreateWithAttributedString((__bridge CFAttributedStringRef)(self.text));
    CGSize size = CTFramesetterSuggestFrameSizeWithConstraints(frameSetter, CFRangeMake(0, self.text.length), nil, CGSizeMake(self.width, CGFLOAT_MAX), nil);
    return size.height;
}

- (void)setText:(NSAttributedString *)text
{
    _text = text;
    [self setNeedsDisplay];
}

- (void)drawRect:(CGRect)rect
{
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextTranslateCTM(context, 0.0, CGRectGetHeight(self.bounds));
    CGContextScaleCTM(context, 1.0, -1.0);
    
    CGContextSetFillColorWithColor(context, [[UIColor clearColor] CGColor]);
    CGContextFillRect(context, self.bounds);
    
    CTFramesetterRef frameSetter = CTFramesetterCreateWithAttributedString((__bridge CFAttributedStringRef)self.text);
    
    if (self.textFrame != nil) {
        CFRelease(self.textFrame);
    }
    
    self.textFrame = CTFramesetterCreateFrame(frameSetter, CFRangeMake(0, 0), self.path, nil);
    CFRetain(self.textFrame);
    
    CTFrameDraw(self.textFrame, context);
    //CGContextRelease(context);
}

- (int)indexAtPoint:(CGPoint)point
{
    point = CGPointMake(point.x, CGRectGetMaxY(self.bounds) - point.y);
    NSArray *lines = (__bridge NSArray *)CTFrameGetLines(self.textFrame);
    
    CGPoint origins[lines.count];
    CTFrameGetLineOrigins(self.textFrame, CFRangeMake(0, lines.count), origins);
    
    for (int i = 0; i < lines.count; i++) {
        CTLineRef line = (__bridge CTLineRef)[lines objectAtIndex:i];
        CGFloat lineWidth = CTLineGetTypographicBounds(line, nil, nil, nil);
        if (point.y > origins[i].y && point.x > origins[i].x && point.x < origins[i].x + lineWidth) {
            return CTLineGetStringIndexForPosition(line, point);
        }
    }
    return 0;
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint touchPoint = [touch locationInView:self];
    int index = [self indexAtPoint:touchPoint];
    
    //NSLog(@"chatacter touched: %d", index);
    NSDictionary *attributes = [self.text attributesAtIndex:index effectiveRange:nil];
    for (NSString* key in attributes) {
        id value = [attributes objectForKey:key];
        [delegate textView:self didSelectAttributeWithType:key Value:value];
    }
}

@end
